package logic

import (
	"encoding/json"
	"errors"
	"fusenapi/constants"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"gorm.io/gorm"
	"strings"

	"context"

	"fusenapi/server/product/internal/svc"
	"fusenapi/server/product/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetSizeByPidLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetSizeByPidLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetSizeByPidLogic {
	return &GetSizeByPidLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *GetSizeByPidLogic) GetSizeByPid(req *types.GetSizeByPidReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	req.Pid = strings.Trim(req.Pid, " ")
	if req.Pid == "" {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "err param:pid is empty")
	}
	//获取产品信息(只是获取id)
	productInfo, err := l.svcCtx.AllModels.FsProduct.FindOneBySn(l.ctx, req.Pid, "id")
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "the product is not exists")
		}
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get product info")
	}
	//获取产品尺寸列表(需要正序排序)
	sizeList, err := l.svcCtx.AllModels.FsProductSize.GetAllByProductIds(l.ctx, []int64{productInfo.Id}, "is_popular DESC,sort ASC")
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get size list")
	}
	sizeIds := make([]int64, 0, len(sizeList))
	for _, v := range sizeList {
		sizeIds = append(sizeIds, v.Id)
	}
	//获取对应模型数据
	modelList, err := l.svcCtx.AllModels.FsProductModel3d.GetAllBySizeIdsTag(l.ctx, sizeIds, constants.TAG_MODEL, "id,size_id")
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get model list")
	}
	mapSizeModel := make(map[int64]int) //size id为key
	for k, v := range modelList {
		mapSizeModel[*v.SizeId] = k
	}
	//处理
	listRsp := make([]types.GetSizeByPidRsp, 0, len(sizeList))
	for _, sizeInfo := range sizeList {
		//没有模型的不能使用
		modelIndex, ok := mapSizeModel[sizeInfo.Id]
		if !ok {
			continue
		}
		var title interface{}
		_ = json.Unmarshal([]byte(*sizeInfo.Title), &title)
		listRsp = append(listRsp, types.GetSizeByPidRsp{
			Id:              sizeInfo.Id,
			Title:           title,
			Capacity:        *sizeInfo.Capacity,
			Cover:           *sizeInfo.Cover,
			PartsCanDeleted: *sizeInfo.PartsCanDeleted > 0,
			ModelId:         modelList[modelIndex].Id,
			IsPopular:       *sizeInfo.IsPopular > 0,
		})
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "success", listRsp)
}
